package com.ylx.massage.utils;

/**
 * @author jianlong
 * @date 2024-06-04 10:13
 */
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson2.JSONObject;
import com.ylx.common.config.WechatAccountConfig;
import com.ylx.common.exception.ServiceException;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Map;



/**
 * 微信登录工具类
 *
 */
@Component
public class WeChatUtil {

    private final static String ERROR_CODE = "errcode";

    @Resource
    private WechatAccountConfig wxPayProperties;


    /**
     * 获取微信授权code
     *
     * @param state state信息
     * @return 返回微信授权code的URL
     */
    public String getCode(String state) {
        try {
            StringBuffer url = new StringBuffer();
            url.append(wxPayProperties.getGetCodeUrl())
                    .append("?appid=")
                    .append(wxPayProperties.getMpAppId())
                    .append("&redirect_uri=")
                    .append(URLEncoder.encode(wxPayProperties.getRedirectUrl(), "UTF-8"))
                    .append("&response_type=code&scope=snsapi_userinfo&state=")
                    .append(state)
                    .append("#wechat_redirect");
            return url.toString();
        } catch (UnsupportedEncodingException e) {
            throw new ServiceException("URL格式化异常");
        }

    }

    /**
     * 获取微信AccessToken
     *
     * @param code 用户code
     * @return 返回包含微信AccessToken的Map
     */
    public Map<?, ?> getAccessToken(String code) {
        StringBuffer url = new StringBuffer();
        url.append(wxPayProperties.getAccessTokenUrl())
                .append("?appid=").append(wxPayProperties.getMpAppId())
                .append("&secret=").append(wxPayProperties.getMpAppSecret())
                .append("&code=").append(code)
                .append("&grant_type=authorization_code");
        String rs = HttpUtil.get(url.toString());
        Map<?, ?> map = JSONObject.parseObject(rs, Map.class);
        if (null == map.get(ERROR_CODE)) {
            return map;
        } else {
            throw new ServiceException("获取access_token出错");
        }
    }

    /**
     * 刷新AccessToken
     *
     * @param refreshToken 用户刷新token
     * @return 返回包含刷新后的微信AccessToken的Map
     */
    public Map<?, ?> refreshToken(String refreshToken) {
        StringBuffer url = new StringBuffer();
        url.append("https://api.weixin.qq.com/sns/oauth2/refresh_token")
                .append("?appid=").append(wxPayProperties.getMpAppId())
                .append("&grant_type=refresh_token&refresh_token=").append(refreshToken);
        String rs = HttpUtil.get(url.toString());
        Map<?, ?> map = JSONObject.parseObject(rs, Map.class);
        if (null == map.get(ERROR_CODE)) {
            return map;
        } else {
            throw new ServiceException("刷新access_token出错");
        }
    }

    /**
     * 获取用户信息
     *
     * @param accessToken 微信AccessToken
     * @param openid 用户的openid
     * @return 返回包含用户信息的JSON字符串
     */
    public String getUserInfo(String accessToken, String openid) {
        StringBuffer url = new StringBuffer();
        url.append("https://api.weixin.qq.com/sns/userinfo")
                .append("?access_token=").append(accessToken)
                .append("&openid=").append(openid)
                .append("&lang=zh_CN");
        return HttpUtil.get(url.toString());
    }
}
